/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: cmd_join.c,v 1.19.4.2 2002/03/05 22:17:32 lloydy Exp $
 *
 */
#include "chunkyircd.h"
#include "client.h"
#include "channel.h"
#include "ban.h"

CMIRCD_MODULE(1.01,CMD_CLIENT);

/* Routines used locally */
static int confirm_join(ClientInfo ,ChannelInfo );
static int join_check_perms(ClientInfo ,ChannelInfo ,char *,int *);

/* 
* cmd_join - deals with join commands 
*/    
int cmd_join(ClientInfo c_ptr,ServerInfo s_ptr,Cmd cmd)
{
	char *channels[7],*keys[7];
	int i,currkey,currchan;
	char buffer[BUF_SIZE];
	ClientInfo origin=NULL;
	ChannelInfo chan=NULL;
	int init_mode=0;

	/* Not valid for a server yet */
	if(s_ptr) {
		 
		return 0;
	}
	
	/* NULL out the key/channels arrays */
	for(i=0;i<7;i++) {
		channels[i]=NULL;
		keys[i]=NULL;
	}
	
	/* Scan through the parameter list, filter out keys and channels */
	for(i=0,currchan=0,currkey=0;cmd->params[i] && currkey < 7 && currchan < 7;i++) {
		if(channel_check(cmd->params[i])) {
			channels[currchan++]=cmd->params[i];
		} else {
			keys[currkey++]=cmd->params[i];
		}
	}
	
	/* Check for at least one valid channel */
	if(!channels[0]) {
		if(c_ptr) {
			ircd_sprintf(buffer,BUF_SIZE,"%s :Invalid Channel mask",cmd->params[0]);
			send_numeric(c_ptr,get_my_server(),476,buffer);
		} 
		 
		return 0;
	}
	
	for(currchan=0,currkey=0;channels[currchan] && currchan < 7;currchan++) {
		init_mode=0;
		chan=NULL;
		chan=locate_channel(channels[currchan]);
		
		/* Run extra checks if they are a local client */
		if(!join_check_perms(c_ptr,chan,keys[currkey],&currkey)) {
			/* The channel permmissions are wrong */
			 
			return 0;
		}
		origin=c_ptr;
		
		/* Actually join here */
		/* Add client to chan and chan to client */
		if(!chan) {
			/* Create channel */
			chan=channel_create(channels[currchan],time(NULL));
			init_mode=CHANUSR_OP;
		}
		client_add_chan(origin,chan);
		channel_add_client(origin,chan,init_mode);
		
		/* Clients like this */
		ircd_sprintf(buffer,BUF_SIZE,"JOIN :%s",chan->name);
		send_client_client(c_ptr,c_ptr,buffer);
		confirm_join(c_ptr,chan);
		send_to_local_channel(origin,chan,buffer);
		
		/* Servers prefer this format */
		ircd_sprintf(buffer,BUF_SIZE,"SJOIN %ld %s 0 :%s%s",chan->ts,chan->name,
				channel_is_client_on(origin,chan)->chan_mode & CHANUSR_OP?"@":"",
				origin->nick);
		servers_relay(s_ptr,s_ptr?s_ptr->name:NULL,buffer);
					
	}
		
	
	 
	return 1;

}	

/*
 * Check the user has the correct permissions to join
 * returns 1 if they can, 0 otherwise
 */
static int join_check_perms(ClientInfo c_ptr,ChannelInfo chan,char *key,int *key_no)
{

	char buffer[BUF_SIZE];

	(*key_no)++;	
	if(chan) {
		
		if(channel_is_client_on(c_ptr,chan))
			return 0;
		
		if(chan->user_count == chan->user_limit) {
			/* Channel Limit Hit */
			ircd_sprintf(buffer,BUF_SIZE,"%s :Cannot join channel (+l)",chan->name);
			send_numeric(c_ptr,get_my_server(),471,buffer);
			return 0;
		}
		if(check_ban(chan,c_ptr)) {
			/* They are banned */
			ircd_sprintf(buffer,BUF_SIZE,"%s :Cannot join channel (+b)",chan->name);
			send_numeric(c_ptr,get_my_server(),474,buffer);
			return 0; 
		}
		if(chan->key) {
			/* Check the channel key matches */
			if( !(key) || (strcmp(chan->key,key)) ) {
				/* No key passed or key doesn't match */ 
				ircd_sprintf(buffer,BUF_SIZE,"%s :Cannot join channel (+k)",chan->name);
				send_numeric(c_ptr,get_my_server(),475,buffer);
				return 0;
			}
		}
				
	} 
	
	return 1;
}

/*
 * Send channel information to the client
 * topic and names lists are sent
 */
static int confirm_join(ClientInfo client,ChannelInfo chan) 
{
	char buffer[BUF_SIZE];
	
	buffer[0]='\0';
	
	/* Send topic */
	if(chan->topic[0]!='\0') {
		/* Send topic */
		ircd_sprintf(buffer,BUF_SIZE,"%s :%s",chan->name,chan->topic);
		send_numeric(client,get_my_server(),332,buffer);
		/* Who set it */
		ircd_sprintf(buffer,BUF_SIZE,"%s %s %ld",chan->name,chan->topic_set,chan->topic_ts);
		send_numeric(client,get_my_server(),333,buffer);
	}

	/* Names list */
	/* in cmd_names.c */
	send_namelists(chan,client);

	/* Need to send end of /names message as well */
	ircd_sprintf(buffer,BUF_SIZE,"%s :End of /NAMES list",chan->name);
	send_numeric(client,get_my_server(),366,buffer);
	return 1;
}
